#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* 
   == Two Element Chain ==

   Command-line args are: 

   * k, the spring constant (m = 1 by definition)
   * mode (1 or 2), specifying the normal mode to initialize
   * ampl, the amplitude of the initial displacement
   * its, the number of time steps to simulate
   * dt, the time step

*/

int main (int argc, char *argv[]) {

  int n = 2;
  int i,  mode, it, maxits;
  double k, om, x[4], v[4], ampl, dt, einit, etot, dt2;
  double tally, om1, om2, a1, a2;
  double k1v, k2v[4], k1x, k2x[4];
  FILE *f1, *f2, *fe;

  /* Arrays have extra points to control the boundary conditions */

  if (argc != 6) {
    printf("Should be five arguments! Exiting...\n");
    exit(2);
  }

  k      = atof(argv[1]);
  mode   = atoi(argv[2]);
  ampl   = atof(argv[3]);
  maxits = atoi(argv[4]);
  dt     = atof(argv[5]);

  dt2 = dt*dt;

  /* Fixed walls: */
  x[0] = x[n+1] = 0.0;

/*   if (mode == 1) { */
/*     x[1] = ampl; */
/*     x[2] = -ampl; */
/*     om = sqrt(3.0*k); */
/*   } */
/*   else if (mode == 2) { */
/*     x[1] = x[2] = ampl; */
/*     om = sqrt(k); */
/*   } */
/*   else { */
/*     printf("Undefined mode! Exiting...\n"); */
/*     exit(1); */
/*   } */

  /* Some general initial displacements */
  x[2] = -.34;
  x[1] = .02;
  a1 = (x[1] - x[2])/2.0;
  a2 = (x[1] + x[2])/2.0;
  om1 = sqrt(3.0*k);
  om2 = sqrt(k);

  /* Initial (t=0) velocities set here: */
  for (i=0; i<n+2; i++)
    v[i] = 0.0;

  /* For half-step method.  This is v[-1/2] = v[0] - (1/2) a[0] dt;
     the first iteration below then produces v[+1/2] as needed. */
  for (i=1; i<=n; i++)
    v[i] += 0.5*k*(2.0*x[i] - x[i-1] - x[i+1])*dt;

  f1 = fopen("mass1", "w");
  f2 = fopen("mass2", "w");
/*   fe = fopen("energy", "w"); */
  fe = fopen("exact", "w");

  /* Calculate initial energy */
/*   einit = 0.0; */
/*   for (i=1; i<=n; i++) { */
/*     /\* Include PE for spring to left of each mass: *\/ */
/*     einit += 0.5*v[i]*v[i] + 0.5*k*pow(x[i]-x[i-1], 2); */
/*     /\* Add in spring to right of last mass: *\/ */
/*     einit += pow(x[n]-x[n+1], 2); */
/*   } */
/*   tally = 0.0; */
/*   etot = einit; */

  /* Main loop */
  for (it = 0; it < maxits; it++) {

    /* Write output */
    if (it % 5 == 0) {
/*       fprintf (fe, "%lf\n", etot); */
      fprintf (fe, "%lf\n", a1*cos(om1*it*dt)+a2*cos(om2*it*dt));
      fprintf (f1, "%lf\n", x[1]);
      fprintf (f2, "%lf\n", x[2]);
    }

    /* Evolution by half step method */
    for (i=1; i<=n; i++)
      v[i] += - k*(2.0*x[i] - x[i-1] - x[i+1])*dt;

    for (i=1; i<=n; i++)
      x[i] += v[i]*dt;

    /* Calculate total energy */
/*     etot = 0.0; */
/*     for (i=1; i<=n; i++) { */
/*       /\* Include PE for spring to left of each mass: *\/ */
/*       etot += 0.5*v[i]*v[i] + 0.5*k*pow(x[i]-x[i-1], 2); */
/*       /\* Add in spring to right of last mass: *\/ */
/*       etot += pow(x[n]-x[n+1], 2); */
/*     } */
/*     tally += pow(etot - einit, 2); */
  }

/*   printf("Average energy error   = %lf\n", sqrt(tally/maxits)); */
/*   printf("Average relative error in energy = %le\n", sqrt(tally/maxits)/einit); */

  fclose (fe);
  fclose (f1);
  fclose (f2);

  return 0;
}
