#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* 
   == N-element chain with driving force ==

   Command-line args are: 

   * N, the number of masses
   * k, the spring constant (m = 1 by definition)
   * ampl, the amplitude of the driving force
   * om_d, the angular frequency of the driving force
   * its, the number of time steps to simulate
   * dt, the time step

*/

int main (int argc, char *argv[]) {

  int n, i,  it, maxits;
  double k, omd, *x, *v, ampl, dt, maxamp, tmp;
  FILE *f1, *f2;

  /* Arrays have extra points to control the boundary conditions */

  if (argc != 7) {
    printf("Should be six arguments! Exiting...\n");
    exit(2);
  }

  n      = atoi(argv[1]);
  k      = atof(argv[2]);
  ampl   = atof(argv[3]);
  omd    = atof(argv[4]);
  maxits = atoi(argv[5]);
  dt     = atof(argv[6]);

  /* Allocate arrays */
  x = (double *) calloc (n + 2, sizeof(double));
  v = (double *) calloc (n + 2, sizeof(double));

  for (i=0; i<=n+1; i++)
    x[i] = 0.0;

  /* TESTING!! */
/*   x[2] = -.34; */
/*   x[1] = .02; */

  /* Initial (t=0) velocities set here: */
  for (i=0; i<n+2; i++)
    v[i] = 0.0;

  /* For half-step method.  This is v[-1/2] = v[0] - (1/2) a[0] dt;
     the first iteration below then produces v[+1/2] as needed. */
  for (i=1; i<=n; i++)
    v[i] += 0.5*k*(2.0*x[i] - x[i-1] - x[i+1])*dt;

  f1 = fopen("mass1", "w");
  f2 = fopen("massN", "w");

  /* For dentifying resonance frequencies: */
/*   maxamp = 0.0; */

  /* Main loop */
  for (it = 0; it < maxits; it++) {

    /* Write output */
    if (it % 5 == 0) {
      fprintf (f1, "%lf\n", x[1]);
      fprintf (f2, "%lf\n", x[n]);
    }

    /* Update driving mass */
    x[0] = ampl*sin(omd*(it+1)*dt);

    /* Evolution by half step method */
    for (i=1; i<=n; i++)
      v[i] += - k*(2.0*x[i] - x[i-1] - x[i+1])*dt;

    for (i=1; i<=n; i++)
      x[i] += v[i]*dt;

/*     if ((tmp = fabs(x[1])) > maxamp)  */
/*       maxamp = tmp; */
  }
  fclose (f1);
  fclose (f2);

/*   printf("Max displacement of mass 1 = %lf\n", maxamp); */

  return 0;
}
