#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define N 50
#define M 51
#define OUTSKIP 1000

/* 
   == N x M element mattress with driving force ==

   Driving mass is located in the middle of the top boundary, for now.

   Command-line args are:

   * k      the spring constant (m = 1 by definition)
   * ampl   the amplitude of the driving force
   * om_d   the angular frequency of the driving force
   * its    the number of time steps to simulate
   * dt     the time step

*/

int main (int argc, char *argv[]) {

  int i, j, it, maxits;
  double k, omd, x[N+2][M+2], v[N+2][M+2];
  double ampl, dt;
  FILE *fp;
  char outbase[5] = "out.";
  char outfile[15];
  char outsuffix[10];

  if (argc != 6) {
    printf("Should be five arguments! Exiting...\n");
    exit(2);
  }

  k      = atof(argv[1]);
  ampl   = atof(argv[2]);
  omd    = atof(argv[3]);
  maxits = atoi(argv[4]);
  dt     = atof(argv[5]);

  for (i=0; i<=N+1; i++)
    for (j=0; j<=M+1; j++)
      x[i][j] = 0.0;

  /* Initial (t=0) velocities set here: */
  for (i=1; i<=N; i++)
    for (j=1; j<=M; j++)
      v[i][j] = 0.0;

  /* For half-step method.  This is v[-1/2] = v[0] - (1/2) a[0] dt;
     the first iteration below then produces v[+1/2] as needed. */
  for (i=1; i<=N; i++)
    for (j=1; j<=M; j++)
      v[i][j] += 0.5*k*(4.0*x[i][j] 
			- x[i-1][j] - x[i+1][j]
			- x[i][j-1] - x[i][j+1])*dt;

  /* Main loop */
  for (it = 0; it < maxits; it++) {

    /* Write output */
    if (it % OUTSKIP == 0) {
      strcpy (outfile, outbase);
      sprintf (outsuffix, "%04d", it/OUTSKIP);
      strcat (outfile, outsuffix);

      fp = fopen(outfile, "w");
      for (i=1; i<=N; i++) {
	for (j=1; j<=M; j++)
	  fprintf (fp, "%lf\n", x[i][j]);
	fprintf (fp, "\n");
      }
      fclose (fp);
    }

    /* Update driving mass(es) */
    x[0][(M+1)/2] = ampl*sin(omd*(it+1)*dt);
/*     x[N+1][(M+1)/2] = ampl*sin(omd*(it+1)*dt); */

    /* Evolution by half step method */
    for (i=1; i<=N; i++)
      for (j=1; j<=M; j++) {
	v[i][j] += - k*(4.0*x[i][j] - x[i-1][j] - x[i+1][j]
			- x[i][j-1] - x[i][j+1])*dt;
      }

    for (i=1; i<=N; i++)
      for (j=1; j<=M; j++)
	x[i][j] += v[i][j]*dt;

  }
  return 0;
}
